@@ -63,7 +63,7 @@ except NameError:
     unicode = str
 
 
-__version__ = '0.9.4'
+__version__ = '0.9.5'
 
 
 CR = '\r'
@@ -122,15 +122,18 @@ def extended_blank_lines(logical_line,
                          indent_level,
                          previous_logical):
     """Check for missing blank lines after class declaration."""
-    if (previous_logical.startswith('class ')):
-        if (logical_line.startswith(('def ', 'class ', '@')) or
-                pep8.DOCSTRING_REGEX.match(logical_line)):
+    if previous_logical.startswith('class '):
+        if (
+            logical_line.startswith(('def ', 'class ', '@')) or
+            pep8.DOCSTRING_REGEX.match(logical_line)
+        ):
             if indent_level and not blank_lines:
-                yield (0, 'E301 expected 1 blank line, found 0')
+                yield (0, 'E309 expected 1 blank line after class declaration')
     elif previous_logical.startswith('def '):
         if blank_lines and pep8.DOCSTRING_REGEX.match(logical_line):
             yield (0, 'E303 too many blank lines ({0})'.format(blank_lines))
     elif pep8.DOCSTRING_REGEX.match(previous_logical):
+        # Missing blank line between class docstring and method declaration.
         if (
             indent_level and
             not blank_lines and
@@ -166,6 +169,7 @@ def continued_indentation(logical_line, tokens, indent_level, noqa):
 
     last_token_multiline = None
     line = None
+    last_line = ''
     for token_type, text, start, end, line in tokens:
 
         newline = row < start[0] - first_row
@@ -205,7 +209,10 @@ def continued_indentation(logical_line, tokens, indent_level, noqa):
                 # visual indent is verified
                 if not indent[depth]:
                     indent[depth] = start[1]
-            elif visual_indent in (text, str):
+            elif (
+                visual_indent in (text, unicode) and
+                not last_line.rstrip().endswith(',')
+            ):
                 # ignore token lined up with matching one from a previous line
                 pass
             elif indent[depth] and start[1] < indent[depth]:
@@ -237,7 +244,7 @@ def continued_indentation(logical_line, tokens, indent_level, noqa):
         # deal with implicit string concatenation
         elif (token_type in (tokenize.STRING, tokenize.COMMENT) or
               text in ('u', 'ur', 'b', 'br')):
-            indent_chances[start[1]] = str
+            indent_chances[start[1]] = unicode
         # special case for the "if" statement because len("if (") == 4
         elif not indent_chances and not row and not depth and text == 'if':
             indent_chances[end[1] + 1] = True
@@ -271,6 +278,7 @@ def continued_indentation(logical_line, tokens, indent_level, noqa):
                 indent_chances[start[1]] = text
 
         last_token_multiline = (start[0] != end[0])
+        last_line = line
 
     if indent_next and pep8.expand_indent(line) == indent_level + 4:
         yield (last_indent, 'E125 {0}'.format(indent_level + 8))
@@ -353,6 +361,7 @@ class FixPEP8(object):
         self.fix_e272 = self.fix_e271
         self.fix_e273 = self.fix_e271
         self.fix_e274 = self.fix_e271
+        self.fix_e309 = self.fix_e301
         self.fix_e703 = self.fix_e702
         self.fix_w191 = self.fix_e101
 
@@ -533,9 +542,11 @@ class FixPEP8(object):
         # When multiline strings are involved, pep8 reports the error as
         # being at the start of the multiline string, which doesn't work
         # for us.
-        if ('"""' in target or
+        if (
+            '"""' in target or
             "'''" in target or
-                target.rstrip().endswith('\\')):
+            target.rstrip().endswith('\\')
+        ):
             return []
 
         fixed = fix_whitespace(target,
@@ -559,8 +570,10 @@ class FixPEP8(object):
 
         # Only proceed if non-whitespace characters match.
         # And make sure we don't break the indentation.
-        if (fixed.replace(' ', '') == target.replace(' ', '') and
-                _get_indentation(fixed) == _get_indentation(target)):
+        if (
+            fixed.replace(' ', '') == target.replace(' ', '') and
+            _get_indentation(fixed) == _get_indentation(target)
+        ):
             self.source[result['line'] - 1] = fixed
         else:
             return []
@@ -854,30 +867,38 @@ class FixPEP8(object):
         target = self.source[line_index]
         offset = result['column'] - 1
 
-        right_offset = offset + 2
-        if right_offset >= len(target):
-            return []
+        # Handle very easy "not" special cases.
+        if re.match(r'^\s*if \w+ == False:$', target):
+            self.source[line_index] = re.sub(r'if (\w+) == False:',
+                                             r'if not \1:', target, count=1)
+        elif re.match(r'^\s*if \w+ != True:$', target):
+            self.source[line_index] = re.sub(r'if (\w+) != True:',
+                                             r'if not \1:', target, count=1)
+        else:
+            right_offset = offset + 2
+            if right_offset >= len(target):
+                return []
 
-        left = target[:offset].rstrip()
-        center = target[offset:right_offset]
-        right = target[right_offset:].lstrip()
+            left = target[:offset].rstrip()
+            center = target[offset:right_offset]
+            right = target[right_offset:].lstrip()
 
-        # Handle simple cases only.
-        new_right = None
-        if center.strip() == '==':
-            if re.match(r'\bTrue\b', right):
-                new_right = re.sub(r'\bTrue\b *', '', right, count=1)
-        elif center.strip() == '!=':
-            if re.match(r'\bFalse\b', right):
-                new_right = re.sub(r'\bFalse\b *', '', right, count=1)
+            # Handle simple cases only.
+            new_right = None
+            if center.strip() == '==':
+                if re.match(r'\bTrue\b', right):
+                    new_right = re.sub(r'\bTrue\b *', '', right, count=1)
+            elif center.strip() == '!=':
+                if re.match(r'\bFalse\b', right):
+                    new_right = re.sub(r'\bFalse\b *', '', right, count=1)
 
-        if new_right is None:
-            return []
+            if new_right is None:
+                return []
 
-        if new_right[0].isalnum():
-            new_right = ' ' + new_right
+            if new_right[0].isalnum():
+                new_right = ' ' + new_right
 
-        self.source[line_index] = left + new_right
+            self.source[line_index] = left + new_right
 
     def fix_w291(self, result):
         """Remove trailing whitespace."""
@@ -917,9 +938,10 @@ def fix_e26(source, _=False):
     fixed_lines = []
     sio = io.StringIO(source)
     for (line_number, line) in enumerate(sio.readlines(), start=1):
-        if (line.lstrip().startswith('#') and
-                line_number not in ignored_line_numbers):
-
+        if (
+            line.lstrip().startswith('#') and
+            line_number not in ignored_line_numbers
+        ):
             indentation = _get_indentation(line)
             line = line.lstrip()
 
@@ -991,7 +1013,6 @@ def fix_w6(source, aggressive=True):
                      'sys_exc',
                      'throw',
                      'tuple_params',
-                     'types',
                      'xreadlines'])
 
 
@@ -1075,6 +1096,8 @@ def _priority_key(pep8_result):
         'e225', 'e231',
         # Remove extraneous whitespace before breaking lines.
         'e201',
+        # Shorten whitespace in comment before resorting to wrapping.
+        'e262'
     ]
     key = pep8_result['id'].lower()
     if key in priority:
@@ -1127,8 +1150,11 @@ def _shorten_line(tokens, source, indentation, indent_word, newline,
         token_string = tkn[1]
 
         if (
+            aggressive and
             token_type == tokenize.COMMENT and
-            not previous_line.rstrip().endswith('\\')
+            not previous_line.rstrip().endswith('\\') and
+            not source[tkn[2][1] + 1:].strip().lower().startswith(
+                ('noqa', 'pragma:', 'pylint:'))
         ):
             # Move inline comments to previous line.
             offset = tkn[2][1]
@@ -1331,7 +1357,7 @@ class Reindenter(object):
 
         """
         try:
-            stats = reindent_stats(tokenize.generate_tokens(self.getline))
+            stats = _reindent_stats(tokenize.generate_tokens(self.getline))
         except (SyntaxError, tokenize.TokenError):
             return set()
         # Remove trailing empty lines.
@@ -1422,8 +1448,8 @@ class Reindenter(object):
         return line
 
 
-def reindent_stats(tokens):
-    """Return list  of (lineno, indentlevel) pairs.
+def _reindent_stats(tokens):
+    """Return list of (lineno, indentlevel) pairs.
 
     One for each stmt and comment line. indentlevel is -1 for comment lines, as
     a signal that tokenize doesn't know what to do about them; indeed, they're
@@ -1594,8 +1620,12 @@ def filter_results(source, results, aggressive=False):
         if issue_id == 'e101' and '\t' not in split_source[r['line']]:
             continue
 
-        if not aggressive:
-            if issue_id.startswith(('e711', 'e712', 'w6')):
+        if aggressive <= 0:
+            if issue_id.startswith(('e711', 'w6')):
+                continue
+
+        if aggressive <= 1:
+            if issue_id.startswith(('e712', )):
                 continue
 
         if r['line'] in commented_out_code_line_numbers:
@@ -1622,7 +1652,7 @@ def multiline_string_lines(source, include_docstrings=False):
             start_row = t[2][0]
             end_row = t[3][0]
 
-            if (token_type == tokenize.STRING and start_row != end_row):
+            if token_type == tokenize.STRING and start_row != end_row:
                 if (
                     include_docstrings or
                     previous_token_type != tokenize.INDENT
@@ -1683,8 +1713,10 @@ def shorten_comment(line, newline, max_line_length, last_comment=False):
                           len(indentation) + 72)
 
     MIN_CHARACTER_REPEAT = 5
-    if (len(line) - len(line.rstrip(line[-1])) >= MIN_CHARACTER_REPEAT and
-            not line[-1].isalnum()):
+    if (
+        len(line) - len(line.rstrip(line[-1])) >= MIN_CHARACTER_REPEAT and
+        not line[-1].isalnum()
+    ):
         # Trim comments that end with things like ---------
         return line[:max_line_length] + newline
     elif last_comment and re.match(r'\s*#+\s*\w+', line):
@@ -1862,13 +1894,13 @@ def extract_code_from_function(function):
     return code
 
 
-def parse_args(args):
-    """Parse command-line options."""
+def create_parser():
+    """Return command-line parser."""
     parser = optparse.OptionParser(usage='Usage: %prog [options] '
                                          '[filename [filename ...]]'
                                          '\nUse filename \'-\'  for stdin.',
                                    version='%prog {0}'.format(__version__),
-                                   description=__doc__.split('\n')[0],
+                                   description=docstring_summary(__doc__),
                                    prog='autopep8')
     parser.add_option('-v', '--verbose', action='count', dest='verbose',
                       default=0,
@@ -1879,8 +1911,8 @@ def parse_args(args):
     parser.add_option('-i', '--in-place', action='store_true',
                       help='make changes to files in place')
     parser.add_option('-r', '--recursive', action='store_true',
-                      help='run recursively; must be used with --in-place or '
-                           '--diff')
+                      help='run recursively over directories; '
+                           'must be used with --in-place or --diff')
     parser.add_option('-j', '--jobs', type=int, metavar='n', default=1,
                       help='number of parallel jobs; '
                            'match CPU count if value is less than 1')
@@ -1892,8 +1924,8 @@ def parse_args(args):
                       help='enable non-whitespace changes; '
                            'multiple -a result in more aggressive changes')
     parser.add_option('--exclude', metavar='globs',
-                      help='exclude files/directories that match these '
-                           'comma-separated globs')
+                      help='exclude files that match these comma-separated '
+                           'globs')
     parser.add_option('--list-fixes', action='store_true',
                       help='list codes for fixes; '
                            'used by --ignore and --select')
@@ -1910,6 +1942,13 @@ def parse_args(args):
                       help='only fix errors found within this inclusive '
                            'range of line numbers (e.g. 1 99); '
                            'line numbers are indexed at 1')
+
+    return parser
+
+
+def parse_args(args):
+    """Parse command-line options."""
+    parser = create_parser()
     options, args = parser.parse_args(args)
 
     if not len(args) and not options.list_fixes:
@@ -1995,13 +2034,20 @@ def supported_fixes():
     for attribute in dir(instance):
         code = re.match('fix_([ew][0-9][0-9][0-9])', attribute)
         if code:
-            yield (code.group(1).upper(),
-                   re.sub(r'\s+', ' ',
-                          getattr(instance, attribute).__doc__))
+            yield (
+                code.group(1).upper(),
+                re.sub(r'\s+', ' ',
+                       docstring_summary(getattr(instance, attribute).__doc__))
+            )
 
     for (code, function) in sorted(global_fixes()):
         yield (code.upper() + (4 - len(code)) * ' ',
-               re.sub(r'\s+', ' ', function.__doc__))
+               re.sub(r'\s+', ' ', docstring_summary(function.__doc__)))
+
+
+def docstring_summary(docstring):
+    """Return summary of docstring."""
+    return docstring.split('\n')[0]
 
 
 def line_shortening_rank(candidate, newline, indent_word):
@@ -2029,8 +2075,10 @@ def line_shortening_rank(candidate, newline, indent_word):
             '{': '}'}.get(lines[0][-1], None)
 
         if len(lines) > 1:
-            if (bad_staring_symbol and
-                    lines[1].lstrip().startswith(bad_staring_symbol)):
+            if (
+                bad_staring_symbol and
+                lines[1].lstrip().startswith(bad_staring_symbol)
+            ):
                 rank += 20
 
         if re.match(r'.*[+\-\*/] \($', lines[0]):
@@ -2044,8 +2092,10 @@ def line_shortening_rank(candidate, newline, indent_word):
 
             for ending in '([{':
                 # Avoid lonely opening. They result in longer lines.
-                if (current_line.endswith(ending) and
-                        len(current_line.strip()) <= len(indent_word)):
+                if (
+                    current_line.endswith(ending) and
+                    len(current_line.strip()) <= len(indent_word)
+                ):
                     rank += 100
 
             if current_line.endswith('%'):
@@ -2146,7 +2196,7 @@ def _fix_file(parameters):
     try:
         fix_file(*parameters)
     except IOError as error:
-        print(str(error), file=sys.stderr)
+        print(unicode(error), file=sys.stderr)
 
 
 def fix_multiple_files(filenames, options, output=None):
